<?php
namespace EMS\PortalFuncionario\Geral\Repository;

use Zeedhi\Framework\Controller\Crud;
use Doctrine\ORM\EntityManager;

use EMS\PortalFuncionario\Util\DateUtil;

class GeralRepository {

    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    public function getAniversariantes(
        int $nrOrg,
            $dtmescompetenc = null
    ): array {
        if (empty($dtmescompetenc)) {
            $dtmescompetenc = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataAtual())->format(DateUtil::FORMATO_BRASILEIRO);
        }

        $sql = <<<SQL
            SELECT DISTINCT
            TO_CHAR(PEH.DTNASCPESSOA, 'DD/MM') DTANIVERSARIO,
            VM.NMVINCULOM,
            EH.NMESTRUTURAH,
            OP.DSEMAILOPER
            FROM GPE_VINCULOM VM
            JOIN GPE_VINCULOH VH ON VH.NRORG      = VM.NRORG
                                AND VH.NRVINCULOM = VM.NRVINCULOM
                                AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                          WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                                            AND NRORG      = VH.NRORG
                                                            AND NRVINCULOM = VH.NRVINCULOM )
            /*Estrutura Gerencial*/
            JOIN ESTRUTURAM EMG ON EMG.NRORG        = VM.NRORG
                               AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN --!
            JOIN ESTRUTURAH EH ON EH.NRORG = EMG.NRORG
                              AND EH.NRESTRUTURAH = (
                                  SELECT EH2.NRESTRUTURAH
                                  FROM ESTRUTURAH EH2
                                  WHERE EH2.NRESTRUTURAM = EMG.NRESTRUTURAM
                                    AND EH2.NRORG = EMG.NRORG
                                    AND EH2.DTMESCOMPETENC = (
                                        SELECT MAX(DTMESCOMPETENC)
                                        FROM ESTRUTURAH EH3
                                        WHERE EH3.NRESTRUTURAM = EMG.NRESTRUTURAM
                                          AND EH3.NRORG = EMG.NRORG
                                    )
                              )
            JOIN GPE_PESSOA PE ON PE.NRORG    = VM.NRORG
                              AND PE.NRPESSOA = VM.NRPESSOA
            JOIN GPE_PESSOAH PEH ON PEH.NRORG    = VM.NRORG
                                     AND PEH.NRPESSOA = PE.NRPESSOA
                                     AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                                                                WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
                                                                  AND NRORG    = PEH.NRORG
                                                                  AND NRPESSOA = PEH.NRPESSOA )
                                    AND SUBSTR(:P_DTMESCOMPETENC, 4, 2) = TO_CHAR(PEH.DTNASCPESSOA, 'MM')
                                    AND PEH.DTNASCPESSOA IS NOT NULL
            -- /*Ocupação*/
            -- LEFT JOIN GPE_OCUPACAOH OH ON OH.NRORG       = VM.NRORG
            --                           AND OH.NROCUPACAOM = NVL( VH.NRCARGO, VH.NRFUNCAO )
            --                           AND OH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_OCUPACAOH
            --                                                     WHERE DTMESCOMPETENC <= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY')
            --                                                       AND NRORG       = OH.NRORG
            --                                                       AND NROCUPACAOM = OH.NROCUPACAOM )
            LEFT JOIN OPERADOR OP ON OP.NRORG = VM.NRORG
                                      AND OP.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
            WHERE VM.NRORG = :P_NRORG
            AND ( VM.DTRESCISAOVINC >= TO_DATE(:P_DTMESCOMPETENC, 'DD/MM/YYYY') OR VM.DTRESCISAOVINC IS NULL )
            ORDER BY DTANIVERSARIO ASC
SQL;

        $params = [
            "P_NRORG"           => $nrOrg,
            "P_DTMESCOMPETENC"  => $dtmescompetenc,
        ];

        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);

        $aniversarios = $std->fetchAll();

        return $aniversarios;
    }
}