<?php

namespace EMS\PortalFuncionario\Util;

use GuzzleHttp\Client;

/**
 *  Classe criada com o proposito de reunir rotinas uteis para a realizacao de requisicoes tanto para o proprio projeto quanto para outros projetos
 * 
 */

class Request {

    /**
     *  Rotina responsavel por retornar o parametro 'baseUri' definido no envirnoment.xml
     * 
     *  @return string
     */
    public static function getBaseUriParameter() {
        return Util::getParameter('baseUri');
    }

    /**
     *  Rotina responsavel por retornar o parametro 'loginUri' definido no envirnoment.xml
     * 
     *  @return string
     */
    public static function getLoginUriParameter() {
        return Util::getParameter('loginUri');
    }

    /**
     *  Rotina responsavel por realizar requisicoes utilizando a dependência 'GuzzleHttp\Client'
     * 
     *  @param  string  $method
     *  @param  string  $uri
     *  @param  array   $options
     * 
     *  @return Array response
     */
    public static function request(string $method, string $uri, array $options) {
        $http = new Client();
        $response = $http->request($method, $uri, $options);
        $content = $response->getBody()->getContents();
        return json_decode($content, true);
    }

    /**
     *  Rotina capaz de retornar a URL do backend do projeto
     * 
     *  @return string
     */
    public static function getProjectUrl() {
        // Build requested url
        $fullRequestedUrl = $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'];

        $baseUrlPos = strpos($fullRequestedUrl, self::getBaseUriParameter());
        if ($baseUrlPos === false) throw new \Exception('Não foi possível obter a URL do projeto.');

        // Build project url
        $requestUrl = substr($fullRequestedUrl, 0, $baseUrlPos);

        return $requestUrl;
    }

    /**
     *  Rotina capaz de adicionar o protocolo no inicio de uma URL
     * 
     *  @param  string  $url
     *  @param  string  $protocol
     * 
     *  @return string
     */
    public static function addProtocolUrl(string $url, string $protocol) {
        return $protocol . '://' . $url;
    }

    /**
     *  Rotina responsavel por retornar a URL requisitavel (com protocolo) para o modulo de login (loginsetup)
     * 
     *  @return string
     */
    public static function getLoginUrl() {
        $requestUrl = self::getProjectUrl();

        $loginUrl = $requestUrl . self::getLoginUriParameter();

        // Maintain request protocol
        $loginUrl = self::addProtocolUrl($loginUrl, $_SERVER['REQUEST_SCHEME']);

        return $loginUrl;
    }

    /**
     *  Rotina capaz de acionar uma rota a partir da rota do modulo de login
     * 
     *  @param  string  $route
     *  @param  array   $options
     * 
     *  @return Array   self::request
     */
    public static function loginRequest(string $route, array $options) {
        $loginRoute = self::getLoginUrl() . $route;

        $response = self::request('POST', $loginRoute, $options);

        return $response;
    }

}