/* Tabela VENDA: Tabela de Vendas */
/* Descrio: A trigger insere registros na tabela LOGOPERA a partir de alteraes na tabela
              VENDA referente ao processo de Fechamento de Cupom */
/* Objetivo: Gerar um log que vai registrar as alteraes na tabela Venda que podem influenciar no
             processo de Fechamento de Cupom, utilizando a tabela Geral de Logs de Operaes */
CREATE OR REPLACE TRIGGER LOG_FIS_VENDA
AFTER DELETE OR INSERT OR UPDATE
ON VENDA FOR EACH ROW
DECLARE
   V_DSREMOTEADDR VARCHAR2(50);
   V_DSREMOTEHOST VARCHAR2(50);
   V_DSSERVERRDM  VARCHAR2(20);
   V_DSSERVMETHOD VARCHAR2(50);
   V_DSREMOTEOPER VARCHAR2(12);
   V_NMOPERADOR   VARCHAR2(100);
   V_TIPO         VARCHAR2(1);
   V_DESC         VARCHAR2(1000);
   V_CDFILLOGOPER VARCHAR2(4);
   V_ITPESQ       VARCHAR2(40);
   V_DTOPER       DATE;
   V_STATUS       VARCHAR2(20);
BEGIN
   V_TIPO := '0';
   V_DESC := '';
   V_CDFILLOGOPER := '';

   IF DELETING THEN
      IF :OLD.IDAPROVACAONFCE = 'A' THEN
         IF :OLD.IDSITUVENDA IS NOT NULL THEN
            V_DESC := V_DESC || 'SituacaoVenda=' || :OLD.IDSITUVENDA;
         END IF;
         IF :OLD.DTEMISSAONFCE IS NOT NULL THEN
            V_DESC := V_DESC || ' DataEmissao=' || :OLD.DTEMISSAONFCE;
         END IF;
         IF :OLD.NRACESSONFCE IS NOT NULL THEN
            V_DESC := V_DESC || ' ChaveAcesso=' || :OLD.NRACESSONFCE;
         END IF;
         IF :OLD.IDTPAMBNFCE IS NOT NULL THEN
            IF :OLD.IDTPAMBNFCE = '1' THEN
               V_STATUS := 'Producao';
            ELSE
               V_STATUS := 'Homologacao';
            END IF;
            V_DESC := V_DESC || ' Ambiente=' || V_STATUS;
         END IF;
         IF :OLD.IDSTATUSNFCE IS NOT NULL THEN
            V_DESC := V_DESC || ' StatusTransmis=' || :OLD.IDSTATUSNFCE;
         END IF;
         IF :OLD.CDSERIESAT IS NOT NULL THEN
            V_DESC := V_DESC || ' SerieSat=' || :OLD.CDSERIESAT;
         END IF;
         IF :OLD.IDLIBERAPROV IS NOT NULL THEN
            V_DESC := V_DESC || ' LiberaAprov=' || :OLD.IDLIBERAPROV;
         END IF;
         IF :OLD.IDTPEMISVEND IS NOT NULL THEN
            V_DESC := V_DESC || ' TipoEmissao=' || :OLD.IDTPEMISVEND;
         END IF;
         IF :OLD.IDTPAPROV IS NOT NULL THEN
            V_DESC := V_DESC || ' TipoAprov=' || :OLD.IDTPAPROV;
         END IF;
      END IF;

      V_TIPO   := '3';
      V_ITPESQ := :OLD.CDFILIAL||:OLD.CDCAIXA||TO_CHAR(:OLD.DTENTRVENDA,'DDMMYYYY')||:OLD.NRSEQVENDA;
   ELSE
      IF (:OLD.IDAPROVACAONFCE = 'A' OR NVL(:OLD.IDAPROVACAONFCE,'!') <> NVL(:NEW.IDAPROVACAONFCE,'!')) THEN
         IF UPDATING THEN
            IF NVL(:NEW.IDSITUVENDA,  '!') <> NVL(:OLD.IDSITUVENDA,  '!') OR
               NVL(:NEW.NRACESSONFCE, '!') <> NVL(:OLD.NRACESSONFCE, '!') OR
               NVL(:NEW.IDTPAMBNFCE,  '!') <> NVL(:OLD.IDTPAMBNFCE,  '!') OR
               NVL(:NEW.IDSTATUSNFCE, '!') <> NVL(:OLD.IDSTATUSNFCE, '!') OR
               NVL(:NEW.IDTPEMISVEND, '!') <> NVL(:OLD.IDTPEMISVEND, '!') OR
               NVL(:NEW.IDTPAPROV,    '!') <> NVL(:OLD.IDTPAPROV,    '!') OR
               NVL(:NEW.DTEMISSAONFCE,TO_DATE('01/01/1900', 'DD/MM/YYYY')) <> NVL(:OLD.DTEMISSAONFCE,TO_DATE('01/01/1900', 'DD/MM/YYYY')) OR
               NVL(:NEW.DTENTRVENDA,  TO_DATE('01/01/1900', 'DD/MM/YYYY')) <> NVL(:OLD.DTENTRVENDA,  TO_DATE('01/01/1900', 'DD/MM/YYYY')) OR
               NVL(:NEW.IDAPROVACAONFCE, '!') <> NVL(:OLD.IDAPROVACAONFCE, '!') OR
               NVL(:NEW.CDSERIESAT,   '!') <> NVL(:OLD.CDSERIESAT,   '!') OR
               NVL(:NEW.IDLIBERAPROV, '!') <> NVL(:OLD.IDLIBERAPROV, '!') THEN
               V_TIPO   := '2';
               V_ITPESQ := :OLD.CDFILIAL||:OLD.CDCAIXA||TO_CHAR(:OLD.DTENTRVENDA,'DDMMYYYY')||:OLD.NRSEQVENDA;
            END IF;
         ELSE
            V_TIPO   := '1';
            V_ITPESQ := :NEW.CDFILIAL||:NEW.CDCAIXA||TO_CHAR(:NEW.DTENTRVENDA,'DDMMYYYY')||:NEW.NRSEQVENDA;
         END IF;

         IF NVL(:NEW.IDSITUVENDA, '!') <> NVL(:OLD.IDSITUVENDA, '!') THEN
            V_DESC := V_DESC || 'SituacaoVenda=' || :NEW.IDSITUVENDA;
         END IF;
         IF NVL(:NEW.NRACESSONFCE, '!') <> NVL(:OLD.NRACESSONFCE, '!') THEN
            V_DESC := V_DESC || ' ChaveAcesso=' || :NEW.NRACESSONFCE;
         END IF;
         IF NVL(:NEW.IDTPAMBNFCE, '!') <> NVL(:OLD.IDTPAMBNFCE, '!') THEN
            IF :NEW.IDTPAMBNFCE = '1' THEN
               V_STATUS := 'Producao';
            ELSE
               V_STATUS := 'Homologacao';
            END IF;
            V_DESC := V_DESC || ' Ambiente=' || V_STATUS;
         END IF;
         IF NVL(:NEW.IDSTATUSNFCE, '!') <> NVL(:OLD.IDSTATUSNFCE, '!') THEN
            V_DESC := V_DESC || ' StatusTransmis=' || :NEW.IDSTATUSNFCE;
         END IF;
         IF NVL(:NEW.IDTPEMISVEND, '!') <> NVL(:OLD.IDTPEMISVEND, '!') THEN
            V_DESC := V_DESC || ' TipoEmissao=' || :NEW.IDTPEMISVEND;
         END IF;
         IF NVL(:NEW.DTEMISSAONFCE,TO_DATE('01/01/1900', 'DD/MM/YYYY')) <> NVL(:OLD.DTEMISSAONFCE,TO_DATE('01/01/1900', 'DD/MM/YYYY')) THEN
            V_DESC := V_DESC || ' DataEmissao=' || :NEW.DTEMISSAONFCE;
         END IF;
         IF NVL(:NEW.DTENTRVENDA,TO_DATE('01/01/1900', 'DD/MM/YYYY')) <> NVL(:OLD.DTENTRVENDA,TO_DATE('01/01/1900', 'DD/MM/YYYY')) THEN
            V_DESC := V_DESC || ' DataEntrVenda=' || :NEW.DTENTRVENDA;
         END IF;
         IF NVL(:NEW.IDAPROVACAONFCE, '!') <> NVL(:OLD.IDAPROVACAONFCE, '!') THEN
            V_DESC := V_DESC || ' IdAprovacao=' || :NEW.IDAPROVACAONFCE;
         END IF;
         IF NVL(:NEW.CDSERIESAT, '!') <> NVL(:OLD.CDSERIESAT, '!') THEN
            V_DESC := V_DESC || ' SerieSat=' || :NEW.CDSERIESAT;
         END IF;
         IF NVL(:NEW.IDLIBERAPROV, '!') <> NVL(:OLD.IDLIBERAPROV, '!') THEN
            V_DESC := V_DESC || ' LiberaAprov=' || :NEW.IDLIBERAPROV;
         END IF;
         IF :OLD.IDTPAPROV IS NOT NULL THEN
            V_DESC := V_DESC || ' TipoAprov=' || :OLD.IDTPAPROV;
         END IF;
      END IF;
   END IF;

   IF  V_TIPO <> '0' THEN
       NMUSUARIO(V_NMOPERADOR);
       WEBTUNNING(V_DSREMOTEADDR,
                  V_DSREMOTEHOST,
                  V_DSSERVERRDM,
                  V_DSSERVMETHOD,
                  V_DSREMOTEOPER);

       IF NOT V_DESC IS NULL THEN
         IF NOT V_NMOPERADOR IS NULL THEN
            V_DTOPER := SYSDATE;
            INSERT INTO LOGOPERA
               (NMTABELA,     DTOPERACAO,   NMRESPOPER,   IDTIPOOPER,  CDPESQITEM,  DSOPERACAO,
                CDFILLOGOPER, DSRDMUSUOPER, DSENDUSUOPER, DSNMUSUOPER, DSMETUSUOPER,CDORESPOPER)
            VALUES
               ('VENDA',   V_DTOPER,      V_NMOPERADOR,   V_TIPO,         V_ITPESQ,       SUBSTR(V_DESC,1,400),
                V_CDFILLOGOPER, V_DSSERVERRDM, V_DSREMOTEADDR, V_DSREMOTEHOST, V_DSSERVMETHOD, V_DSREMOTEOPER);
         END IF;
       END IF;
   END IF;
END;
/

-- VERSAO --
BEGIN ATUVEROBJ('LOG_FIS_VENDA','6.06.000'); END;
/
